/**
 * Created by mintao on 16/5/26.
 */

var FYMath = FYMath || {};

FYMath.M_PI = 3.14159265358979323846264338327950288;/* pi */
FYMath.M_PI_2 = 1.57079632679489661923132169163975144;/* pi/2 */
/***
 * 获取随机数
 * @returns {*}
 * @constructor
 */
FYMath.getRandom = function (Min, Max){
    var Range = Max - Min;
    var Rand = Math.random();// 结果为0-1间的一个随机数(包括0,不包括1)
    var random = Min + Math.round(Rand * Range);//round函数结果为num四舍五入后的整数
    return random;
};

/***
 * 保留几位小数, 不四舍五入
 * @param num
 * @param count
 * @returns {string}
 * @constructor
 */
FYMath.fixedFloor = function( num , multiple , index ){
    num = (Math.floor(num /multiple * (index * 10))).toFixed(index)/ (index * 10);
    return num ;
};


/***
 * 计算两点的距离
 */
FYMath.getDistance = function(x1,y1,x2,y2){
    return Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
};

/***
 * 两点的弧度
 * @returns {number}
 */
FYMath.getRadian = function(x1,y1,x2,y2){
    var x = Math.abs(x2 - x1);
    var y = Math.abs(y2 - y1);
    var z = FYMath.getDistance(x1,y1,x2,y2);
    if (z == 0){
        return 0;
    }
    var sin = x/z;
    return Math.asin(sin);
};

/***
 * 两点的角度
 * @returns {number}
 */
FYMath.getAngle = function(x1,y1,x2,y2){
    var distance = FYMath.getDistance(x1,y1,x2,y2);
    if (distance == 0){
        return 0;
    }

    var angle = 90-Math.atan2(y2-y1,x2-x1)*180/Math.PI;
    return angle;

    //var radian = FYMath.getRadian(x1,y1,x2,y2);
    //if (radian == 0){
    //    return 0;
    //}
    //
    //var angle = 180/(Math.PI/radian);
    //var x = x2 - x1;
    //var y = y2 - y1;
    //if (x > 0 && y < 0){
    //    angle = 180-angle;
    //}
    //else if (x < 0 && y < 0){
    //    angle += 180;
    //}
    //else if (x < 0 && y > 0){
    //    angle = 360-angle;
    //}
    //return angle;
};

/***
 * 角度转弧度
 * @param angle
 * @returns {number}
 * @constructor
 */
FYMath.AngletoRadian= function(angle){
    return Math.PI/180*angle;
};

/**
 * 通过距离和角度,计算出x的长度
 * @param z 三角形第三边长度
 * @param angle
 */
FYMath.getSinx = function(z, angle){
    var radian = FYMath.AngletoRadian(angle);
    return Math.sin(radian)*z;
};

/**
 * 通过距离和角度,计算出x的长度
 * @param z 三角形第三边长度
 * @param angle
 */
FYMath.getCosx = function(z, angle){
    var radian = FYMath.AngletoRadian(angle);
    return Math.cos(radian)*z;
};


/**
 * 阶乘
 * @param n
 */
FYMath.Factorial = function(number)
{
    var factorial = 1;
    var temp = number;
    for (var i = 0; i < number; ++ i)
    {
        factorial *= temp;
        -- temp;
    }

    return factorial;
};

FYMath.Combination = function( count,  r)
{
    return parseInt(FYMath.Factorial(count) / (FYMath.Factorial(r) * FYMath.Factorial(count - r)));
};

/****
 *
 * @param initX
 * @param initY
 * @param initCount
 * @param fDistance
 * @returns {Array}
 * @constructor
 */
FYMath.BuildBezier = function(initX, initY, initCount, fDistance)
{
    cc.log('-----------FYMath.BuildBezier');
    var traceVector = [];
    var index = 0;
    var tPos = cc.p(0,0);
    var t = 0;
    var count = initCount - 1;
    var tempValue = 0;

    while (t <= 1) {
        tPos = cc.p(0,0);
        index = 0;
        while (index <= count) {
            tempValue = Math.pow(t, index) * Math.pow(1 - t, count - index) * FYMath.Combination(count, index);
            tPos.x += initX[index] * tempValue;
            tPos.y += initY[index] * tempValue;
            ++ index;
        }

        var fSpace = 0;
        if (traceVector.length > 0) {
            var backPos = traceVector[traceVector.length-1];
            fSpace = FYMath.getDistance(backPos.x, backPos.y, tPos.x, tPos.y);
        }

        if (fSpace >= fDistance || traceVector.length == 0) {
            traceVector.push(tPos);
        }

        t += 0.001;
    }

    cc.log('-----------FYMath.BuildBezier1111111');
    return traceVector;
};

FYMath.BuildBezierFast = function(initX, initY, initCount, fDistance)
{
    var traceVector = [];
    var index = 0;
    var tPos = cc.p(0,0);
    var t = 0;
    var count = initCount - 1;
    var tfDis = fDistance;
    var tempValue = 0;

    while (t <= 1)
    {
        tPos = cc.p(0,0);
        index = 0;
        while (index <= count)
        {
            tempValue = Math.pow(t, index) * Math.pow(1 - t, count - index) * FYMath.Combination(count, index);
            tPos.x += initX[index] * tempValue;
            tPos.y += initY[index] * tempValue;
            ++ index;
        }

        var fSpace = 0;
        if (traceVector.length > 0) {
            var backPos = traceVector[traceVector.length-1];
            fSpace = FYMath.getDistance(backPos.x, backPos.y, tPos.x, tPos.y);
            cc.log('---------fSpace :'+fSpace);
        }

        if (fSpace >= tfDis || traceVector.length == 0) {
            traceVector.push(tPos);
        }

        t += 0.01;
    }

    return traceVector;
};


FYMath.BuildBezierWithAngle = function(initX, initY, initCount, fDistance){
    var traceVector = new Array();//[];
    var pos1 = new CMovePoint();
    traceVector.push(pos1);

    var index = 0;
    var tPos0 = new CMovePoint();
    var t = 0;
    var count = initCount - 1;
    var tfDis = fDistance;
    var tPos = new CMovePoint();
    var tempValue = 0;

    while (t < 1)
    {
        tPos.m_Position.x += 0;
        tPos.m_Position.y += 0;
        index = 0;
        while (index <= count)
        {
            tempValue = Math.pow(t, index) * Math.pow(1 - t, count - index) * FYMath.Combination(count, index);
            tPos.m_Position.x += initX[index] * tempValue;
            tPos.m_Position.y += initY[index] * tempValue;
            ++index;
        }

        var fSpace = 0;
        if (traceVector.length > 0)
        {
            var backPos = traceVector[traceVector.length-1];
            fSpace = FYMath.getDistance(backPos.m_Position.x, backPos.m_Position.y, tPos.m_Position.x, tPos.m_Position.y);
        }

        if (fSpace >= tfDis || traceVector.length == 0)
        {
            if (traceVector.length > 0)
            {
                var temp_dis = FYMath.getDistance(tPos.m_Position.x, tPos.m_Position.y, tPos0.m_Position.x, tPos0.m_Position.y);
                if (temp_dis != 0)
                {
                    var tempValue = (tPos.m_Position.x - tPos0.m_Position.x) / temp_dis;
                    if ((tPos.m_Position.y - tPos0.m_Position.y) >= 0)
                        tPos.m_Direction = Math.acos(tempValue);
                    else
                        tPos.m_Direction = -Math.acos(tempValue);
                }
                else
                {
                    tPos.m_Direction = 1;
                }
            }
            else
            {
                tPos.m_Direction = 1;
            }
            //FYEngine.log(tPos, '-----------tPos : ');
            var tempPos = new CMovePoint();
            tempPos.setValue(tPos);
            traceVector.push(tempPos);
            tPos0.m_Position.x = tPos.m_Position.x;
            tPos0.m_Position.y = tPos.m_Position.y;
        }

        t += 0.001;
    }
    traceVector.shift();
    //FYEngine.logObject(traceVector, '-----------traceVector : ');
    return traceVector;
};

/***
 * 保留几位小数, 不四舍五入 V1 version
 * @param num
 * @param count
 * @returns {string}
 * @constructor
 */
FYMath.fixedNum = function(num, count){
    var temp = num.toFixed(count);
    temp.substring(0, temp.lastIndexOf('.')+count);
    return temp;
};

/**
 * 数字补 0
 * @param num
 * @param median 位数
 */
FYMath.patchZero= function (num, median) {
    var tmp = "000000000000000000";
    median = median || 2;
    num = num + "";
    var ns = num.length;
    if (ns >= median) {
        return num;
    }
    return tmp.substring(0, median - ns) + num;
};

FYMath.formatDate=function (date, fmt) {
    var that = this;
    fmt = fmt || "yyyy-MM-dd HH:mm:ss";
    return fmt.replace("yyyy", date.getFullYear()).
    replace("MM", that.patchZero(date.getMonth()+1)).
    replace("dd", that.patchZero(date.getDate())).
    replace("HH", that.patchZero(date.getHours())).
    replace("mm", that.patchZero(date.getMinutes())).
    replace("ss", that.patchZero(date.getSeconds())).
    replace("sss", that.patchZero(date.getMilliseconds()));
};

FYMath.formatTimestamp=function (ctime,fmt) {
    var that = this;
    fmt = fmt || "yyyy-MM-dd HH:mm:ss";
    var date = new Date();
    date.setTime(parseInt(ctime)*1000);
    return FYMath.formatDate(date,fmt);
};

FYMath.formatRecountDate=function (ctime) {

    var hor = Math.floor(ctime/3600);
    var min = Math.floor((ctime-3600*hor)/60);
    var sec = ctime - 3600*hor-min*60;
    if(hor<10){
        hor = "0"+hor;
    }
    if(min<10){
        min ="0"+min;
    }
    if(sec<10){
        sec = "0"+sec;
    }
    return ""+hor+":"+min+":"+sec;
};

/**
 *   格式 3：00
 * @param ctime单位秒
 * @returns {string}
 */
FYMath.formatRecountDateShort=function (mtime) {
    var ctime = mtime;
    if(ctime>0){
        var min = Math.floor(ctime/60);
        var sec = Math.floor(ctime-60*min);
        if(sec<10){
            sec = "0"+sec;
        }
        return ""+min+"/"+sec;
    }else{
        return "0/00";
    }

};


//FYMath.formatTimestampRecount=function (ctime,fmt) {
//    var that = this;
//    fmt = fmt || "HH:mm:ss";
//    var hor = Math.floor(ctime/3600);
//    var min = Math.floor((ctime-3600*hor)/60);
//    var sec =
//
//};
/**
 * 闰年
 * @param year
 * @returns {boolean}
 */
FYMath.isLeapYear = function(year) {
    return (year % 400 == 0) || (year % 4 == 0 && year % 100 != 0);
};

FYMath.getMonthDays = function(year, month){
    return [31, null, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month] || (FYMath.isLeapYear(year) ? 29 : 28);
};

FYMath.getWeek = function(ctime){//秒
    var date = new Date();
    date.setTime(parseInt(ctime)*1000);
    var year = date.getFullYear();
    var month = date.getMonth();
    var days = date.getDate();

    for (var i = 0; i < month; i++) {
        days += FYMath.getMonthDays(year, i);
    }
    var yearFirstDay = new Date(year, 0, 1).getDay() || 7;
    var week = null;
    if (yearFirstDay == 1) {
        week = Math.ceil(days / yearFirstDay);
    } else {
        days -= (7 - yearFirstDay + 1);
        week = Math.ceil(days / 7) + 1;
    }

    return week;
};

FYMath.getRecountTime = function(endtime){
    var recount = {};
    var date = new Date();
    var cursec = parseInt(date/1000);
    var range = endtime-cursec;
    return range;

};


/**
 * 从数组中获取不重复的随机数字
 * @param arr
 * @param num
 * @returns {Array}
 */
FYMath.getArrayItems = function(arr, num){
    var temp_array = [];

    for(var t =0;t<arr.length;++t){
        temp_array.push(arr[t]);
    }

    //取出的数值项,保存在此数组
    var return_array = [];
    for (var i = 0; i<num; i++) {
        //判断如果数组还有可以取出的元素,以防下标越界
        if (temp_array.length>0) {
            //在数组中产生一个随机索引
            var arrIndex = Math.floor(Math.random()*temp_array.length);
            //将此随机索引的对应的数组元素值复制出来
            return_array.push(temp_array[arrIndex]);
            //然后删掉此索引的数组元素,这时候temp_array变为新的数组
            temp_array.splice(arrIndex, 1);
        } else {
            //数组中数据项取完后,退出循环,比如数组本来只有10项,但要求取出20项.
            break;
        }
    }
    return return_array;
};

/**
 * 从指定长度中获取不重复的随机数字
 * @param arrlength
 * @param num
 * @returns {Array}
 */
FYMath.getArrayItemsByLength = function(arrlength, num){

    var temp_array = [];
    if(num>arrlength){
        return temp_array;
    }
    for(var t =0;t<arrlength;++t){
        temp_array.push(t);
    }
    if(num==arrlength){
        return temp_array;
    }

    //取出的数值项,保存在此数组
    var return_array = [];
    for (var i = 0; i<num; i++) {
        //判断如果数组还有可以取出的元素,以防下标越界
        if (temp_array.length>0) {
            //在数组中产生一个随机索引
            var arrIndex = Math.floor(Math.random()*temp_array.length);
            //将此随机索引的对应的数组元素值复制出来
            return_array.push(temp_array[arrIndex]);
            //然后删掉此索引的数组元素,这时候temp_array变为新的数组
            temp_array.splice(arrIndex, 1);
        } else {
            //数组中数据项取完后,退出循环,比如数组本来只有10项,但要求取出20项.
            break;
        }
    }
    return return_array;
};

/**
 * 对一个数组进行乱序操作
 * @param arr
 * @returns {Array}
 */

FYMath.mixArrayItem = function(arr){
    var temp_array = [];
    for(var t =0;t<arr.length;++t){
        temp_array.push(arr[t]);
    }

    if(temp_array.length>0){
        for(var i =0,len = temp_array.length;i<len;++i){
            var rand = parseInt(Math.random()*len);
            var temp = temp_array[rand];
            temp_array[rand] = temp_array[i];
            temp_array[i] = temp;
        }
    }

    return temp_array
};


