/**
 * Created by mintao on 16/6/6.
 */

/***
 * loading 转圈动作
 * @returns {*}
 */
FYEngine.loadingAni = function(){
    var rotate1 = new cc.RotateBy(0.5, 360);
    var rotate2 = new cc.RotateBy(0.3, 360);
    var seq = new cc.Sequence(rotate1, rotate2);
    return new cc.RepeatForever(seq);
};

/***
 * 3d 波浪特效
 */
FYEngine.createWaves3D = function(texture){
    var ground = new cc.Sprite(texture);
    var nodeGrid = new cc.NodeGrid();
    nodeGrid.addChild(ground);
    var waves = new cc.Waves3D(10, cc.size(15, 15), 6, 4);//参数的含义 分别是 执行时间，切分的网格大小，波浪次数，波浪大小
    nodeGrid.runAction(new cc.RepeatForever(waves));
    return nodeGrid;
};

/***
 * 液体流动特效
 * @param texture
 * @returns {cc.NodeGrid}
 */
FYEngine.createLiquid = function(texture){
    var ground = new cc.Sprite(texture);
    var nodeGrid = new cc.NodeGrid();
    nodeGrid.setAnchorPoint(cc.p(0,0));
    nodeGrid.addChild(ground);
    var waves = new cc.Liquid(10, cc.size(15, 15), 6, 2);//时间，切分的网格大小，次数，大小
    nodeGrid.runAction(new cc.RepeatForever(waves));
    return nodeGrid;
};

/***
 * 淡入淡出
 */
FYEngine.createFadeAni = function(){
    var fade1 = new cc.FadeTo(0.8, 50);
    var fade2 = new cc.FadeTo(0.5, 255);
    var seq = new cc.Sequence(fade1, fade2);
    return new cc.RepeatForever(seq);
};

/***
 * 创建帧动画
 */
FYEngine.createFrameAni = function(name, count, time){
    var frames = [];
    for(var i = 0; i < count; ++ i){
        var framename = name.format(i);
        var frame = cc.spriteFrameCache.getSpriteFrame(framename);
        frames.push(frame);
    }
    var animation = new cc.Animation(frames, time/count, 1);
    animation.setRestoreOriginalFrame(true);
    var animate = new cc.Animate(animation);
    return animate;
};


FYEngine.createAnimate = function(name){
    var animation = cc.AnimationCache.getInstance().getAnimation(name);
    if (animation == null){
        cc.log('---------animation is null');
        return null;
    }

    return new cc.Animate(animation);
};
/**
 * 弹出动画
 * @param target node
 * @param bigTime 变大时间
 * @param smallTime 回缩时间
 */
FYEngine.popupLayerAnimate = function(target,bigTime,smallTime,position){
    var size = cc.winSize;
    bigTime =bigTime||0.15;
    position = position || cc.p(size.width/2,size.height/2);
    smallTime = smallTime||0.1;
    var animateArr = [];
    if(target){
        //target.setPosition(startPos);
        target.setScale(0.1);
        animateArr.push(cc.targetedAction(target,cc.spawn(cc.targetedAction(target,cc.sequence(cc.scaleTo(bigTime,1),
            cc.scaleTo(0.04,1.1),cc.scaleTo(0.04,1))),cc.moveTo(bigTime,position))));
        animateArr.push(cc.scaleTo(smallTime,1));
        target.runAction(cc.sequence(animateArr));
    }
};
/**
 * 弹出动画
 * @param target
 * @param type
 */
FYEngine.popoutLayerAnimate = function(target,type){
    var size = cc.winSize;
    var animateArr = [];
    if(target){
        //target.setPosition(startPos);
        //animateArr.push(cc.scaleTo(0.1,1.1));
        animateArr.push(cc.targetedAction(target,cc.spawn(cc.scaleTo(0.1,0.1),cc.moveTo(0.1,cc.p(size.width/2,size.height/2)))));
        animateArr.push(cc.callFunc(function(){
            if(type&&type==1){
                target.removeFromParent(true);
            }else{
            target.getParent().removeFromParent(true);
            }
        }));
        target.runAction(cc.sequence(animateArr));
    }
};


var SoftReplaceLength1 = ["收","售","买","卖","赌","送","输","杀","坑","癌","兌","現","苋"];
var SoftReplaceLength2 = ["商人","商行","钱庄","商店","商铺","银商","银行","钱库","银库","网络","送分","上分",
    "下分","赠送","返水","返点","优惠","甩卖","价格","差价","交易","弹头","金币","送芬","兑现","返现","送币","全家",
"户口","骗子","官方","破产","坐牢","天台","扳本","公司","报警","自杀","后台","跳楼","缺德","举报","回本",
"回血","客服","上下","发货","转账","转帐","划账","划帐"];

var SoftReplaceLength3 =["开发商"];
var SoftReplaceLength4 =["家破人亡","倾家荡产","妻离子散","一无所有","断子绝孙"];

function SoftReplaceWords(text){
    var test = text;

    if(test.length>=4){
        for(var m =0;m<SoftReplaceLength4.length;++m){
            var reg4 = new RegExp(""+SoftReplaceLength4[m],"g");
            test = test.replace(reg4,"********");
        }
    }

    if(test.length>=3){
        for(var k =0;k<SoftReplaceLength3.length;++k){
            var reg3 = new RegExp(""+SoftReplaceLength3[k],"g");
            test = test.replace(reg3,"******");
        }
    }

    if(test.length>=2){
        for(var j =0;j<SoftReplaceLength2.length;++j){
            var reg2 = new RegExp(""+SoftReplaceLength2[j],"g");
            test = test.replace(reg2,"****");
        }
    }

    for(var i = 0;i<SoftReplaceLength1.length;++i){
        var reg1 = new RegExp(""+SoftReplaceLength1[i],"g");
        test = test.replace(reg1,"**");
    }

    return test;

};


