/**
 * Created by mintao on 16/5/25.
 * 引擎按钮
 * 单张图片按钮
 */

/***
 * 按钮动作类型
 * @type {{
 * None: number,
 * Scale: number,
 * Fade: number,
 * Dark: number,
 * ScaleDark: number 缩放和变暗
 * }}
 */
FYEngine.ButtonAction = {
    None : -1,
    Scale : 0,
    Fade : 1,
    Dark : 2,
    ScaleDark : 3,
};

FYEngine.SingleButton = ccui.ImageView.extend({
    _actionType : FYEngine.ButtonAction.Scale,
    _labsprite : null, //文本图片
    _callfunc : null, //回调函数

    /****
     * 单张图片按钮
     * @param btnname 按钮图片
     * @param labname 可传undefined或者''
     * @param callaction 按钮回调
     * @param actiontype 可不传
     */
    ctor : function(btnname, labname, callaction, actiontype){
        this._super();

        if (actiontype != undefined){
            this._actionType = actiontype;
        }

        if (callaction != undefined){
            this._callfunc = callaction;
        }

        var texturename = btnname;
        if (btnname[0] == '#'){
            texturename = btnname.slice(1, btnname.length);
            this.loadTexture(texturename, ccui.Widget.PLIST_TEXTURE);
        }
        else {
            this.loadTexture(texturename, ccui.Widget.LOCAL_TEXTURE);
        }

        this.initLabSprite(labname);

        this.setTouchEnabled(true);
        this.addTouchEventListener(this.BtnCallBack);
    },

    initLabSprite : function(labname){
        if (labname == undefined || labname == ''){
            return;
        }

        var size = this.getContentSize();
        this._labsprite = new cc.Sprite(labname);
        this._labsprite.attr({
            x : size.width/2, y : size.height/2,
            anchorX : 0.5, anchorY : 0.5,
        });
        this.addChild(this._labsprite);
    },

    BtnCallBack : function(sender, type){
        switch (type){
            case ccui.Widget.TOUCH_BEGAN:
            {
                sender.btnActionToPress();
            }
                break;
            case ccui.Widget.TOUCH_CANCELED:
            {
                sender.btnActionToNormal();
            }
                break;
            case ccui.Widget.TOUCH_ENDED:
            {
                sender.btnActionToNormal();
                sender._callfunc(sender);
            }
                break;
            default:
                break;
        }
    },

    /**
     * 恢复normal状态
     */
    btnActionToNormal : function(){
        switch (this._actionType){
            case FYEngine.ButtonAction.Fade:
                this.setOpacity(255);
                break;
            case FYEngine.ButtonAction.Dark:
                this.setColor(new cc.Color(180, 167, 167, 255));
                break;
            case FYEngine.ButtonAction.Scale:
                this.setScale(1);
                break;
            default:
                break;
        }
    },

    /**
     * 切换至点击状态
     */
    btnActionToPress : function(){
        switch (this._actionType){
            case FYEngine.ButtonAction.Fade:
                this.setOpacity(180);
                break;
            case FYEngine.ButtonAction.Dark:
                this.setColor(new cc.Color(255, 255, 255, 255));
                break;
            case FYEngine.ButtonAction.Scale:
                this.setScale(0.92);
                break;
            default:
                break;
        }
    },
});


/***
 * @param return ccui.Button
 */
FYEngine.createButton = function(normal, press, pos, callback, anch, disable, textureType){
    if (disable == undefined){
        disable = '';
    }
    if (anch == undefined){
        anch = cc.p(0.5, 0.5);
    }
    if (textureType == undefined){
        textureType = ccui.Widget.PLIST_TEXTURE;
    }

    var btn = new ccui.Button(normal, press, disable, textureType);
    btn.setPosition(pos);
    btn.setAnchorPoint(anch);
    btn.addClickEventListener(callback);
    return btn;
};
