/**
 * Created by mintao on 16/5/26.
 */

FYEngine.listenerList = {};
/****
 * 添加手机按键监听
 * @param target 监听目标
 * @param callfunc 回调监听函数
 * @constructor
 */
FYEngine.AddListenPhoneKey = function(target, func){
    var listener = cc.EventListener.create({
        event : cc.EventListener.KEYBOARD,
        onKeyPressed : function(keyCode, event){
            //cc.log('------------onKeyPressed : ', keyCode);
        },

        onKeyReleased : function (keyCode, event){
            cc.log('------------onKeyReleased : ', keyCode);
            if (func){
                func(keyCode, target);
            }
        },
    });
    cc.eventManager.addListener(listener, target);
};

/****
 * 添加静态事件监听
 * @param target
 * @param name
 * @param func
 * level 优先级
 * @constructor
 */
FYEngine.AddCustomLister = function(target, name, func, nodeOrPriority){
    //if(FYEngine.listenerList&&FYEngine.listenerList[name]){
    //    cc.eventManager.removeListener(FYEngine.listenerList[name])
    //}
    var listener = cc.EventListener.create({
        event : cc.EventListener.CUSTOM,
        eventName : name,
        callback : func.bind(target)
    });

    if (nodeOrPriority==undefined){
        nodeOrPriority = target;
    }
    cc.eventManager.addListener(listener, nodeOrPriority);
};

FYEngine.dispatchCustomEvent = function(eventName, eventParam){
    cc.eventManager.dispatchCustomEvent(eventName, eventParam);
};


/***
 * 添加node点击事件
 * @param target
 * @param func
 * @constructor
 */
FYEngine.AddNodeClickListener = function(node, func){
    var listener = cc.EventListener.create({
        event : cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        onTouchBegan: function (touch, event) {
            var target = event.getCurrentTarget();
            var pos = touch.getLocation();
            if(cc.rectContainsPoint(target.getBoundingBox(), pos)) {
                return true;
            }
            return false;
        },

        onTouchEnded: function (touch, event) {
            var target = event.getCurrentTarget();
            var pos = touch.getLocation();
            if (cc.rectContainsPoint(target.getBoundingBox(), pos)) {
                func(node);
            }
        },
    });
    cc.eventManager.addListener(listener, node);
};


/***
 * 添加node触摸事件
 * @param node
 * @param func
 * @constructor
 */
FYEngine.AddNodeTouchListener = function(node, func, actiontype){
    var listener = cc.EventListener.create({
        event : cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        /**
         * 切换至普通状态
         */
        btnActionToNormal : function(sender, type){
            switch (type){
                case FYEngine.ActionType.Fade:
                    sender.setOpacity(255);
                    break;
                case FYEngine.ActionType.Dark:
                    sender.setColor(new cc.Color(255, 255, 255, 255));
                    break;
                case FYEngine.ActionType.Scale:
                    sender.setScale(1);
                    break;
                default:
                    break;
            }
        },

        /**
         * 切换至点击状态
         */
        btnActionToPress : function(sender, type){
            switch (type){
                case FYEngine.ActionType.Fade:
                    sender.setOpacity(180);
                    break;
                case FYEngine.ActionType.Dark:
                    sender.setColor(new cc.Color(180, 167, 167, 255));
                    break;
                case FYEngine.ActionType.Scale:
                    sender.setScale(0.92);
                    break;
                default:
                    break;
            }
        },

        onTouchBegan: function (touch, event) {
            var target = event.getCurrentTarget();
            if (!target.isVisible()){
                return false;
            }
            var pos = target.convertToNodeSpace(touch.getLocation());
            var size = target.getContentSize();
            if (cc.rectContainsPoint(cc.rect(0, 0, size.width,size.height), pos)) {
                this.btnActionToPress(target, actiontype);
                return true;
            }
            return false;
        },

        onTouchCancelled : function(touch, event){
            var target = event.getCurrentTarget();
            if (!target.isVisible()){
                return;
            }
            this.btnActionToNormal(target, actiontype);
        },

        onTouchEnded: function (touch, event) {
            var target = event.getCurrentTarget();
            if (!target.isVisible()){
                return;
            }
            var pos = target.convertToNodeSpace(touch.getLocation());
            var size = target.getContentSize();
            this.btnActionToNormal(target, actiontype);
            if (cc.rectContainsPoint(cc.rect(0, 0, size.width,size.height), pos)) {
                func(target);
            }
        },
    });
    cc.eventManager.addListener(listener, node);
};

