/**
 * Created by mintao on 16/5/25.
 * 创建引擎label
 */

/****
 * 创建label
 * @param str
 * @param fontsize
 * @param color
 * @param anch
 * @param size 区域大小
 * @param font 字体
 */
FYEngine.createLabel = function(str, fontsize, color, anch, size, font){
    if (fontsize == undefined){
        fontsize = 22;
    }
    if (color == undefined){
        color = cc.color.WHITE;
    }
    if (anch == undefined){
        anch = cc.p(0, 0.5);
    }
    if (font == undefined){
        font = '';
    }

    var lab = cc.Label.createWithSystemFont(str, font, fontsize);
    lab.setColor(color);
    lab.setAnchorPoint(anch);
    if (size != undefined){
        lab.setDimensions(size.width, size.height);
    }

    return lab;
};

/***
 * 创建艺术字体
 * @param filename 图片名字
 * @param count 图片中字体个数
 * @param str 文本
 * @param firstnum 首字符
 */
FYEngine.createAlts = function(filename, count, str, firstnum){
    var textureCache = cc.director.getTextureCache();
    var texture = textureCache.getTextureForKey(filename);
    if (texture == undefined){
        textureCache.addImage(filename);
    }
    texture = textureCache.getTextureForKey(filename);
    var size = texture.getContentSize();
    var width = size.width/count;
    var lab = cc.Label.createWithCharMap(filename, width, size.height, getAscii(firstnum));
    lab.setString(str);

    return lab;
};


/***
 * 截取规则 [phoneu] [/phoneu]
 * @param text 文本
 * @param color1 基础颜色
 * @param color2 特殊颜色
 * @returns {*} 富文本对象
 */
FYEngine.createRichText = function(text, color1, color2, fontsize){
    if (color1 == undefined){
        color1 = cc.color.WHITE;
    }

    if (color2 == undefined){
        color2 = cc.color.YELLOW;
    }

    if (fontsize == undefined){
        fontsize = 25;
    }

    var startKey = "[pu]", endKey = "[/pu]";
    var startlen = startKey.length, endlen = endKey.length;
    var richtext = new ccui.RichText();
    richtext.ignoreContentAdaptWithSize(false);
    var eleIndex = 1;
    while (text.length > 0){
        var startPos = text.indexOf(startKey);
        var endPos = text.indexOf(endKey);
        //cc.log(" startPos is : ", startPos, "endPos is " + endPos);
        if(startPos == -1){
            var eletext = new ccui.RichElementText(eleIndex, color1, 255, text, "", fontsize);
            richtext.pushBackElement(eletext);
            break;
        }
        else if(startPos == 0){
            var eletext = new ccui.RichElementText(eleIndex, color2, 255, text.substr(startlen, endPos-startlen), "", fontsize);
            richtext.pushBackElement(eletext);
            text = text.substr(endPos+endlen);
        }
        else if(startPos >= 1){
            var eletext = new ccui.RichElementText(eleIndex, color1, 255, text.substr(0, startPos), "", fontsize);
            richtext.pushBackElement(eletext);
            text = text.substr(startPos);
        }
        ++ eleIndex;
    }
    return richtext;
};

/***
 * 添加富文本
 * @param target
 * @param richtext
 * @param size
 */
FYEngine.addRichText = function(target, richtext, pos, size, anch){
    richtext.setPosition(pos);
    richtext.setContentSize(size);
    richtext.setAnchorPoint(anch);
    target.addChild(richtext);
};

/***
 *创建有加粗或者带阴影字体的文本
 * @pram str文本内容
 * @pram font字体名
 * @pram fontsize字体尺寸
 * @pram color加粗的颜色
 * @pram size加粗的尺寸
 * @pram shadowColor 阴影的颜色
 * @pram shadowoffset 阴影的偏移 cc.Size
 */


FYEngine.createText = function(str, font, fontsize, size, color, shadowColor, shadowoffset){
    if(font == undefined){
        font = '微软雅黑';
    }
    if(fontsize == undefined){
        fontsize = 22;
    }

    if(size == undefined){
        size = 0;
    }

    if(shadowColor == undefined){
        shadowColor=cc.color(0,0,0,0)
    }
    if(shadowoffset == undefined){
        shadowoffset=cc.size(0,0)
    }
    var label = new ccui.Text(str, font, fontsize);
    if(color ==undefined){
        color =label.getTextColor();
    }

    label.enableOutline(color,size);
    label.enableShadow(shadowColor,shadowoffset,10);
    return label;

};

/***
 * 显示 用户财富 label 大厅专用
 * @param label
 * @param num
 * @param count
 * @param name
 */
FYEngine.createMillionSprite = function (label,num,count,name,str,filename) {
    name=name||"million_icon";count=count||1;num=parseInt(num);str=str||"";
    if(label){
        var _num = num;
        if(num > 100000){
            num = FYMath.fixedFloor(num,10000,count);
        }
        label.setString( str+num );
        if(_num > 100000){
            var sprite =label.getChildByName(name);
            if(sprite)sprite.removeFromParent(true);
            sprite = new cc.Sprite(filename);
            label.addChild(sprite,0,name);
            sprite.attr({x:label.width+sprite.width*0.51, y:label.height*0.5});
        }
    }
    return ( str+num);
};

FYEngine.createLengthText = function(str,size,len){
    var myLabel = new ccui.Text(str, "微软雅黑", size);
    //myLabel.setColor(cc.color("#6b3110"));
    myLabel.attr({x: 180, y: 90, anchorX: 0});
    if(myLabel.getContentSize().width>len){
        var newstr = str;
        var newstr2 = str+"...";
        myLabel.setString(newstr2);
        while(myLabel.getContentSize().width>len){
            newstr = newstr.substring(0,newstr.length-1);
//            cc.log("---------标签的长度大"+newstr);
            var newstr2 = newstr+"...";
            myLabel.setString(newstr2);
        }
        myLabel.setString(newstr2);
    }

    return myLabel;

};

FYEngine.createLengthText2 = function(str,size,len){
    var myLabel = new ccui.Text(str, "微软雅黑", size);
    //myLabel.setColor(cc.color("#6b3110"));
    myLabel.attr({x: 180, y: 90});
    if(myLabel.getContentSize().width>len){
        var newstr = str;
        var newstr2 = str+"...";
        myLabel.setString(newstr2);
        while(myLabel.getContentSize().width>len){
            newstr = newstr.substring(0,newstr.length-1);
//            cc.log("---------标签的长度大"+newstr);
            var newstr2 = newstr+"...";
            myLabel.setString(newstr2);
        }
        myLabel.setString(newstr2);
    }

    return myLabel;

};
FYEngine.NewLengthText2  = ccui.Text.extend({
    _str : "",
    _size: 22,
    _len :0,
    ctor : function(str,size,len){
        this._str = str;
        this._size = size;
        this._len = len;
        this._super(this._str, "", this._size);
        this.shortFunction();
    },

    setStringNew : function (str) {
        this._str = str;
        this.setString(""+str);
    },

    shortFunction : function(){
        if(this.getContentSize().width>this._len){
            var newstr = this._str;
            var newstr2 = this._str+"...";
            this.setString(newstr2);
            while(this.getContentSize().width>this._len){
                newstr = newstr.substring(0,newstr.length-1);
//            cc.log("---------标签的长度大"+newstr);
                var newstr2 = newstr+"...";
                this.setString(newstr2);
            }
            this.setString(newstr2);
        }
    },
});