/**
 * Created by mintao on 16/5/25.
 * 引擎打印功能
 */

/***
 * 打印object所有属性值
 * @param obj
 */
FYEngine.log = function(obj){
    var description = "";
    for(var i in obj){
        var property = obj[i];
        description += i + " = " + property + "\n";
    }
    cc.log(description);
};

/***
 * object转字符串
 * @param arg
 * @param replacer
 * @param space
 * @returns {*}
 */
FYEngine.formatString = function(arg, replacer, space){
    if (typeof arg === 'object') {
        try {
            return JSON.stringify(arg, replacer, space);
        } catch (err) { return ""; }
    } else {return arg; }
};


FYEngine.logObject = function(obj, des){
    if (des == undefined){
        des = '';
    }
    cc.log(des+FYEngine.formatString(obj));
};


/**
 * 15位身份证号码转换为18位
 * @param id   身份证号
 * @returns {boolean}
 * @constructor
 */
FYEngine.Change15To18=function (id) {
    if(id.length!==15){
        return false;
    }

    var zone=id.substring(0,6);
    var year="19" + id.substring(6,8); //
    var mdo=id.substring(8,15);
    id = zone + year + mdo;

    var getNum=parseInt(id.charAt(0)*7+id.charAt(1)*9+id.charAt(2)*10+id.charAt(3)*5+id.charAt(4)*8+id.charAt(5)*4+id.charAt(6)*2+id.charAt(7)*1+id.charAt(8)*6+id.charAt(9)*3+id.charAt(10)*7+id.charAt(11)*9+id.charAt(12)*10+id.charAt(13)*5+id.charAt(14)*8+id.charAt(15)*4+id.charAt(16)*2);

    var lastNumber;
    getNum=getNum%11;
    switch (getNum) {
        case 0 :
            lastNumber="1";
            break;
        case 1 :
            lastNumber="0";
            break;
        case 2 :
            lastNumber="X";
            break;
        case 3 :
            lastNumber="9";
            break;
        case 4 :
            lastNumber="8";
            break;
        case 5 :
            lastNumber="7";
            break;
        case 6 :
            lastNumber="6";
            break;
        case 7 :
            lastNumber="5";
            break;
        case 8 :
            lastNumber="4";
            break;
        case 9 :
            lastNumber="3";
            break;
        case 10 :
            lastNumber="2";
            break;
    }
    var result =id + lastNumber;
    return result.length===18 ? result : false;
};


FYEngine.getAgeByIdCard = function (cardId) {
//获取年龄
    var myDate = new Date();
    var month = myDate.getMonth() + 1;
    var day = myDate.getDate();
    var age = myDate.getFullYear() - parseInt(cardId.substring(6, 10)) - 1;
    if (parseInt(cardId.substring(10, 12)) < month || parseInt(cardId.substring(10, 12)) === month && parseInt(cardId.substring(12, 14)) <= day) {
        age++;
    }
    return age;
};

var vcity={ 11:"北京",12:"天津",13:"河北",14:"山西",15:"内蒙古",
    21:"辽宁",22:"吉林",23:"黑龙江",31:"上海",32:"江苏",
    33:"浙江",34:"安徽",35:"福建",36:"江西",37:"山东",41:"河南",
    42:"湖北",43:"湖南",44:"广东",45:"广西",46:"海南",50:"重庆",
    51:"四川",52:"贵州",53:"云南",54:"西藏",61:"陕西",62:"甘肃",
    63:"青海",64:"宁夏",65:"新疆",71:"台湾",81:"香港",82:"澳门",91:"国外"
};

FYEngine.checkIsRealCard = function(cardId){
    if(cardId === '')
    {
        FYEngine.AddCommToast("身份证号不能为空");
        return false;
    }

    if(FYEngine.IDisCardNo(cardId) === false)
    {
        FYEngine.AddCommToast('您的身份证号码不符合规范，请重新输入');
        return false;
    }
    //检查省份
    if(FYEngine.IDcheckProvince(cardId) === false)
    {
        FYEngine.AddCommToast('您的身份证号码不符合规范，请重新输入');
        return false;
    }
    //校验生日
    if(FYEngine.IDcheckBirthday(cardId) === false)
    {
        FYEngine.AddCommToast('您的身份证号码不符合规范，请重新输入');
        return false;
    }
    //检验位的检测
    if(FYEngine.IDcheckParity(cardId) === false)
    {
        FYEngine.AddCommToast('您的身份证号码不符合规范，请重新输入');
        return false;
    }

    return true;

};

FYEngine.IDisCardNo = function(card)
{
    //身份证号码为15位或者18位，15位时全为数字，18位前17位为数字，最后一位是校验位，可能为数字或字符X
    var reg = /(^\d{15}$)|(^\d{17}(\d|X)$)/;
    if(reg.test(card) === false)
    {
        return false;
    }

    return true;
};

//取身份证前两位,校验省份
FYEngine.IDcheckProvince = function(card)
{
    var province = card.substr(0,2);
    if(vcity[province] == undefined)
    {
        return false;
    }
    return true;
};

//检查生日是否正确
FYEngine.IDcheckBirthday = function(card)
{
    var len = card.length;
    //身份证15位时，次序为省（3位）市（3位）年（2位）月（2位）日（2位）校验位（3位），皆为数字
    if(len == '15')
    {
        var re_fifteen = /^(\d{6})(\d{2})(\d{2})(\d{2})(\d{3})$/;
        var arr_data = card.match(re_fifteen);
        var year = arr_data[2];
        var month = arr_data[3];
        var day = arr_data[4];
        var birthday = new Date('19'+year+'/'+month+'/'+day);
        return FYEngine.IDverifyBirthday('19'+year,month,day,birthday);
    }
    //身份证18位时，次序为省（3位）市（3位）年（4位）月（2位）日（2位）校验位（4位），校验位末尾可能为X
    if(len == '18')
    {
        var re_eighteen = /^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X)$/;
        var arr_data = card.match(re_eighteen);
        var year = arr_data[2];
        var month = arr_data[3];
        var day = arr_data[4];
        var birthday = new Date(year+'/'+month+'/'+day);
        return FYEngine.IDverifyBirthday(year,month,day,birthday);
    }
    return false;
};

//校验日期
FYEngine.IDverifyBirthday = function(year,month,day,birthday)
{
    var now = new Date();
    var now_year = now.getFullYear();
    //年月日是否合理
    if(birthday.getFullYear() == year && (birthday.getMonth() + 1) == month && birthday.getDate() == day)
    {
        //判断年份的范围（0岁到100岁之间)
        var time = now_year - year;
        if(time >= 0 && time <= 100)
        {
            return true;
        }
        return false;
    }
    return false;
};

//校验位的检测
FYEngine.IDcheckParity = function(card)
{
    //15位转18位
    card = FYEngine.IDchangeFivteenToEighteen(card);
    var len = card.length;
    if(len == '18')
    {
        var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
        var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
        var cardTemp = 0, i, valnum;
        for(i = 0; i < 17; i ++)
        {
            cardTemp += card.substr(i, 1) * arrInt[i];
        }
        valnum = arrCh[cardTemp % 11];
        if (valnum == card.substr(17, 1))
        {
            return true;
        }
        return false;
    }
    return false;
};

//15位转18位身份证号
FYEngine.IDchangeFivteenToEighteen = function(card)
{
    if(card.length == '15')
    {
        var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
        var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
        var cardTemp = 0, i;
        card = card.substr(0, 6) + '19' + card.substr(6, card.length - 6);
        for(i = 0; i < 17; i ++)
        {
            cardTemp += card.substr(i, 1) * arrInt[i];
        }
        card += arrCh[cardTemp % 11];
        return card;
    }
    return card;
};

