/****************************************************************************
 Copyright (c) 2008-2010 Ricardo Quesada
 Copyright (c) 2011-2012 cocos2d-x.org
 Copyright (c) 2013-2014 Chukong Technologies Inc.

 http://www.cocos2d-x.org

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ****************************************************************************/

FYEngine.pool = /** @lends cc.pool# */{
    _pool: {},

    _releaseCB: function () {
        this.release();
    },

    _autoRelease: function (obj) {
        var running = obj._running === undefined ? false : !obj._running;
        cc.director.getScheduler().scheduleCallbackForTarget(obj, this._releaseCB, 0, 0, 0, running)
    },

    /***
     * 添加进缓存池 添加tag
     * @param obj
     * @param tag
     */
    putInPoolbyTag : function(obj, tag){
        var pid = obj.constructor.prototype.__pid;
        if (!pid) {
            var desc = { writable: true, enumerable: false, configurable: true };
            desc.value = ClassManager.getNewID();
            Object.defineProperty(obj.constructor.prototype, '__pid', desc);
        }


        // JSB retain to avoid being auto released
        obj.retain && obj.retain();
        // User implementation for disable the object
        obj.unuse && obj.unuse();

        if (!this._pool[pid]) {
            this._pool[pid] = new Map();
        }

        var tagobjs = this._pool[pid].get(tag);
        if (tagobjs == null || tagobjs == undefined){
//            cc.log('-----------这个类的这个tag还没有缓存池 : '+tag);
            tagobjs = new Array();
            this._pool[pid].put(tag, tagobjs);
        }
        tagobjs.push(obj);
    },


    /***
     * 通过tag检测对象
     * @param objClass
     * @param tag
     * @returns {boolean}
     */
    hasObjectbyTag: function (objClass, tag) {
        var pid = objClass.prototype.__pid;
        if (!this._pool[pid] || this._pool[pid].length == 0){
            return false;
        }
        var list = this._pool[pid].get(tag);
        if (list == undefined || !list || list.length == 0) {
            return false;
        }

        return true;
    },

    /**
     * Remove the obj if you want to delete it;
     * @param obj
     */
    removeObject: function (obj) {
        var pid = obj.constructor.prototype.__pid;
        if (pid) {
            var list = this._pool[pid];
            if (list) {
                for (var i = 0; i < list.length; i++) {
                    if (obj === list[i]) {
                        // JSB release to avoid memory leak
                        obj.release && obj.release();
                        list.splice(i, 1);
                    }
                }
            }
        }
    },


    /***
     * 通过tag获取obj
     * 一个tag会有多个obj
     * 默认第一个参数为tag
     * @param objClass
     */
    getFromPoolbyTag : function(objClass/*,args*/){
        var args = Array.prototype.slice.call(arguments);
        args.shift();
        var tag = args[0];
        if (this.hasObjectbyTag(objClass, tag)) {
            var pid = objClass.prototype.__pid;
            var list = this._pool[pid].get(tag);

            var obj = list.shift();
            // User implementation for re-enable the object
            obj.reuse && obj.reuse.apply(obj, args);
            // JSB release to avoid memory leak
            //cc.sys.isNative && obj.release && this._autoRelease(obj);
            return obj;
        }

        return null;
    },

    /**
     *  remove all objs in pool and reset the pool
     */
    drainAllPools: function () {
        //cc.log('--------看看pool:'+this._pool);
        for (var i = 0; i < this._pool.length; ++ i) {
            for (var j = 0; j < this._pool[i].length; j++) {
                var listobj = this._pool[i][j];
                if (listobj){
                    // JSB release to avoid memory leak
                    if (Object.prototype.toString.call(listobj) == '[object Array]'){
                        for (var k = 0; k < listobj.length; ++ k){
                            var obj = listobj[k];
                            obj.release && obj.release();
                        }
                    }
                    else{
                        listobj.release && listobj.release();
                    }
                }
            }
        }
        this._pool = {};
    }
};