/**
 * Created by mintao on 16/6/1.
 */

FYEngine.runScene = function(scene){
    cc.director.runScene(scene);
};

FYEngine.getRunningScene = function(){
    return cc.director.getRunningScene();
};

FYEngine.getRunningLayer = function(tag){
    var scene = cc.director.getRunningScene();
    if (isString(tag)){
        return scene.getChildByName(tag);
    }
    else{
        return scene.getChildByTag(tag);
    }
};

FYEngine.setRunningLayer = function(child, zorder, tag){
    var scene = cc.director.getRunningScene();
    scene.addChild(child, zorder, tag);
};

/***
 * 获取子项
 * @param target
 * @param tag
 * @returns cc.Node
 */
FYEngine.getChild = function(target, tag){
    if (!target)
        return null;

    if (isString(tag)){
        return target.getChildByName(tag);
    }

    if (isNumber(tag))
        return target.getChildByTag(tag);
};

/***
 * 可获取子项的子项
 * @param target
 * @param tag
 * @returns ccui.widget
 */
FYEngine.getWidget = function(target, tag){
    if (!target)
        return null;

    if (isString(tag)){
        return ccui.helper.seekWidgetByName(target, tag);
    }

    if (isNumber(tag)){
        return ccui.helper.seekWidgetByTag(target, tag);
    }
};

/***
 * 向target添加对象child
 * @param child
 * @param target
 * @param pos
 * @param anch 锚点
 * @param zorder 图层
 * @param name or tag : str | int
 */
FYEngine.addChildToTarget = function (child, target, pos, anch, zorder, name) {
    if (child == undefined || target == undefined) {
        return;
    }
    child.setPosition(pos);
    child.setAnchorPoint(anch);
    if (zorder == undefined) {
        zorder = 0;
    }

    if (name == undefined) {
        target.addChild(child, zorder);
        return;
    }

    target.addChild(child, zorder, name);
};

/***
 * loading屏蔽层
 * 转圈
 * @returns {*}
 */
FYEngine.createShieldLayer = function(loading){
    var size = cc.winSize;
    var layer = new cc.LayerColor(cc.color(0, 0, 0, 180), size.width, size.height);
    var touchListener = cc.EventListener.create({
        event: cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        onTouchBegan: function (touch, event) {
            return true;
        }
    });
    cc.eventManager.addListener(touchListener, layer);

    var _sprite = new cc.Sprite(loading);
    if(_sprite){
        _sprite.setPosition(cc.p(layer.width/2, layer.height/2));
        layer.addChild(_sprite);
    }
    _sprite.runAction(new cc.RepeatForever(new cc.RotateBy(1,360)));

    return layer;
};

/***
 * loading层
 * 转圈 + 浮动
 * return layer*
 */
FYEngine.createLoadingLayer = function(loading, anisp){
    var size = cc.winSize;
    var layer = new cc.LayerColor(cc.color(0, 0, 0, 180), size.width, size.height);
    var touchListener = cc.EventListener.create({
        event: cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        onTouchBegan: function (touch, event) {
            return true;
        }
    });
    cc.eventManager.addListener(touchListener, layer);

    var _sprite = new cc.Sprite(loading);
    if(_sprite){
        _sprite.setPosition(cc.p(layer.width/2, layer.height/2));
        layer.addChild(_sprite);
    }
    _sprite.runAction(cc.repeatForever(cc.rotateBy(1,360)));

    var _sprite2 = new cc.Sprite(anisp);
    if(_sprite2){
        _sprite2.setPosition(cc.p(layer.width/2, layer.height/2));
        layer.addChild(_sprite2);
    }
    _sprite2.runAction(new cc.RepeatForever(cc.sequence(cc.moveBy(1,cc.p(0,10)),cc.moveBy(1,cc.p(0,-10)))));
    return layer;
};

/***
 * 获取帧率倍数
 * @returns {number}
 */
FYEngine.getGameRate = function(){
    var cur = cc.director.getAnimationInterval();
    return cur*60;
};

FYEngine.schedule = function(target, callback, interval){
    var then = Date.now();
    interval = interval*1000;
    target.schedule(function(){
       var now = Date.now();
        var delta = now-then;
        if (delta > interval){
            then = now - (delta%interval);
            callback.call(target);
        }
    }.bind(target), 0);
};


FYEngine.CreateAccRelatedLoading = function(){
    var size = cc.winSize;
    var layer = new cc.LayerColor(cc.color(0, 0, 0, 50), size.width, size.height);
    var touchListener = cc.EventListener.create({
        event: cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        onTouchBegan: function (touch, event) {
            return true;
        }
    });
    cc.eventManager.addListener(touchListener, layer);
    var blackbg = new cc.Sprite("#fish_acbind_16.png");
    if(blackbg){
        layer.addChild(blackbg);
        blackbg.setPosition(cc.p(layer.width/2,layer.height/2));

        var _sprite = new cc.Sprite("#fish_acbind_15.png");
        if(_sprite){
            _sprite.setPosition(cc.p(blackbg.width/2, blackbg.height/2));
            blackbg.addChild(_sprite);
        }
        _sprite.runAction(cc.repeatForever(cc.rotateBy(2,360)));
    }

    return layer

};