/**
 * Created by mintao on 16/4/28.
 * 敏感字检测
 */


/***
 * 敏感字文件
  * @type {{File1: string}}
 */
var SensitiveFilePath = {
    File1 : 'res/languageString/sensitiveword.txt',
};


////敏感词
// DFA 实现敏感词过滤 // improve from http://blog.sina.com.cn/s/blog_53f6148101016wq1.html
FYEngine.FilterWord = cc.Class.extend({
    m_wordList : [], // @private
    m_map: null, // @private

    ctor: function () {
        this.m_wordList = this.loadFile(SensitiveFilePath.File1);
        this.m_map = this.BuildMap(this.m_wordList);
    },

    /***
     * 加载敏感字文件
     * @param filepath
     * @returns {Array}
     */
    loadFile : function(filepath){
        var wordlist = [];
        cc.loader.loadTxt(filepath, function(err, data){
            if (err){
                cc.log('--------load failed');
            }
            else{
                wordlist = data.split(/[,\n]/);
            }
        });
        return wordlist;
    },

    /***
     * 替换 str 中的敏感词为 replaceChar,
     * step 为筛选强度， 最小为 0，最大为 5
     * @param str
     * @param replaceChar
     * @param step
     * @returns {*}
     * @constructor
     */
    RunFilterWord: function (str, replaceChar, step) {
        if (str == undefined) {
            return "";
        }

        if (step > 5) {
            step = 5;
        }
        if (step < 0) {
            step = 0;
        }
        for (var j = 0; j < step; ++ j) {
            var result = this.Check(this.m_map, str);
            // 排序结果，长度长的在前面
            result.sort(function(a,b){
                return b.length - a.length});

            cc.log("result: " + result + ', result length : ' + result.length);
            for (var i = 0; i < result.length; i++) {
                var wordlen = result[i].length;
                var replaceStr = '';
                for (var k = 0; k < wordlen; ++ k){
                    replaceStr += replaceChar;
                }
                str = str.replace(result[i], replaceStr);
            }
        }

        return str;
    },

    // @private
    /***
     * 创建敏感词map
     * @param wordList
     * @returns {{}}
     * @constructor
     */
    BuildMap: function (wordList) {
        var result = {};
        var count = wordList.length;
        cc.log('---------敏感词count: ', count);
        // 遍历单词
        for (var i = 0; i < count; ++ i) {
            var map = result;
            var word = wordList[i].replace('\r', '');
//            cc.log('-----敏感词word:', word);
            var wordLength = word.length;

            // 遍历单词的每个字母
            for (var j = 0; j < wordLength; ++ j) {
                var ch = word.charAt(j);
                var stateInfo = map[ch];
                if (stateInfo == null) {
                    stateInfo = {};
                    map[ch] = stateInfo;
                }

                // 如果是最后一个字母，设置一个完结标识
                if (j == wordLength - 1) {
                    stateInfo["isFinish"] = true;
                }
                else {
                    map = stateInfo;
                }
            }
        }

        return result;
    },

    // @private //
    /***
     * 找出 content 中的敏感词
     * 返回一个包含敏感词的列表
     * @param map
     * @param content
     * @returns {Array}
     * @constructor
     */
    Check: function (map, content) {
        var result = [];
        var count = content.length;
        var stack = [];
        var point = map;
        var isFound = false;//用于标记找到关键词的标记
        var foundStack = null;

        for (var i = 0; i < count; ++ i) {
            var ch = content.charAt(i);
            var item = point[ch];
            // 如果没找到则复位，让主循环
            if (item == null) {
                if (isFound) {
                    isFound = false;
                    i = i - (stack.length - foundStack.length + 1); // 计算回退距离
                    result.push(foundStack.join("")); // 把单个的字母数组连成一串字符串
                }
                else {
                    i = i - stack.length; // 计算回退距离
                }
                stack = []; // 清空字符堆栈
                point = map;
            }
            else if (item["isFinish"]) {
                stack.push(ch);
                point = item;
                isFound = true;//标记找到了目标词
                foundStack = stack.concat(); // 复制数组
            }
            else {
                stack.push(ch);
                point = item;
            }
        }
        // 这里还要补充检查
        if (isFound) {
            result.push(foundStack.join("")); // 把单个的字母数组连成一串字符串
        }
        return result;
    },

    /***
     * 销毁敏感字内存
     * @constructor
     */
    Destroy: function () {
        this.m_wordList = [];
        this.m_map = null;
    },

    /***
     * test
     * @param wordList
     * @returns {{}}
     */
    TestbuildMap : function(wordList) {
        var result = {};
        var count = wordList.length;
        for (var i = 0; i < count; ++i) {
            var map = result;
            var word = wordList[i];
            for (var j = 0; j < word.length; ++j) {
                var ch = word.charAt(j);
                if (typeof(map[ch]) != "undefined") {
                map = map[ch];
                if (map["empty"]) {
                    break;}
                }
                else {
                    if (map["empty"]) {
                        delete map["empty"];
                    }
                    map[ch] = {"empty":true};
                    map = map[ch];
                }
            }
        }
        return result;
    },

    /***
     * 测试检测
     * @param map
     * @param content
     * @returns {Array}
     */
    testCheck : function(map, content) {
        var result = [];
        var count = content.length;
        var stack = [];
        var point = map;
        for (var i = 0; i < count; ++ i) {
            var ch = content.charAt(i);
            var item = point[ch];
            if (typeof(item) == "undefined") {
                i = i - stack.length;
                stack = [];
                point = map;
            }
            else if (item["empty"]) {
                stack.push(ch);
                result.push(stack.join(""));
                stack = [];
                point = map;
            }
            else {
                stack.push(ch);
                point = item;
            }
        }
        return result;
    },
});

FYEngine.s_filterWord = null;
FYEngine.FilterWord.getInstance = function(){
    if (!FYEngine.s_filterWord){
        FYEngine.s_filterWord = new FYEngine.FilterWord();
    }
    return FYEngine.s_filterWord;
};

FYEngine.filter = FYEngine.FilterWord.getInstance();
