/**
 * Created by mintao on 16/4/26.
 * toast提示
 */

/***
 * 动作类型: 因为c++项目写了很多但是只用到了2种,这里就不复写了
  * @type {{Bottom: number, Middle: number}}
 */
var ToastActionType = {
    Bottom : 0, //下方弹出
    Middle : 1, //中间打开
};

FYEngine.Toast = ccui.ImageView.extend({
    _actionType : 0,

    /****
     *
     * @param filename 背景图片
     * @param text 提示文本
     * @param actionType toast动作
     * @param rect 拉伸矩形
     * @param textureType 资源类型
     * @param 保留优先级,因为觉得没什么用处,就不加了
     */
    ctor : function(filename, text, actionType, rect, textureType){
        this._super();

        if (textureType == undefined){
            textureType = ccui.Widget.PLIST_TEXTURE;
        }
        if (actionType == undefined){
            actionType = ToastActionType.Bottom;
        }
        this._actionType = actionType;

        this.loadTexture(filename, textureType);
        this.setScale9Enabled(true);
        if (rect != undefined){
            this.setCapInsets(rect);
        }

        if (this.height <= 30){
            this.height += 10;
        }

        var fontsize = 26;
        if (fontsize < 20){
            fontsize = 20;
        }
        var labtext = cc.Label.createWithSystemFont(text, '', fontsize);
        labtext.setAnchorPoint(cc.p(0.5,0.5));

        this.addChild(labtext, 0, 11);
        this.setContentSize(cc.size(labtext.width + 70, this.height));
        labtext.setPosition(cc.p(this.width/2, this.height/2));
        return true;
    },

    /****
     * 获取toast文本
     * @returns {*}
     */
    getLabelText : function(){
        var labtext = this.getChildByTag(11);
        return labtext.getString();
    },

    /****
     * 获取toast动作类型
     * @returns {number}
     */
    getActionType : function(){
        return this._actionType;
    },
});


/***
 * toast管理类
 * @type {Function}
 */
FYEngine.ToastManager = cc.Node.extend({
    _toastList : [],

    ctor : function(){
        this._super();
        this._toastList = [];
    },

    /***
     * 清除toast队列
     */
    clearToastList : function(){
        this._toastList = [];
    },

    /***
     * 创建toast并添加
     * @param filename
     * @param text
     * @param actionType
     * @param rect
     * @param textureType
     * @constructor
     */
    AddTextToToastList : function(filename, text, actionType, rect, textureType){
        var toast = new FYEngine.Toast(filename, text, actionType, rect, textureType);
        if (toast){
            this.AddToastToToastList(toast);
        }
    },

    /***
     * 添加toast
     * @param toast
     * @constructor
     */
    AddToastToToastList : function(toast){
        if (toast == undefined || toast == null){
            return;
        }

        var text = toast.getLabelText();
        if (text == ''){
            return;
        }

        if (this.checkSameToast(text)){
            return;
        }

        //在c++中倒是不用,我也不明白在js中为何需要加retain.
        //但是不加.jsbings的地方就会报错,招不到native object
        //但是我猜想,在c++中我用的是cocos的数据类型Vector,可能他本身就retain过了
        toast.retain();
        this._toastList.push(toast);

        this.playToast();
    },

    /***
     * 播放toast
     */
    playToast : function(){
        if (this._toastList.length == 0){
            return;
        }

        var curscene = cc.director.getRunningScene();
        var curtoast = curscene.getChildByName("AppToast");
        if (curtoast){
            return;
        }

        var toast = this._toastList.shift();
        curscene.addChild(toast, 999, "AppToast");

        var delay = new cc.DelayTime(2);
        var remove = new cc.RemoveSelf();
        var self = this;
        var call = new cc.CallFunc(function(){
            self.playToast();
        });

        var size = cc.winSize;
        var seq;
        switch (toast.getActionType()){
            case ToastActionType.Bottom:
            {
                toast.setPosition(cc.p(size.width/2, size.height/2-100));
                toast.setOpacity(50);
                var fadein = new cc.FadeIn(0.03);
                var moveto1 = new cc.MoveTo(0.1, cc.p(size.width/2, size.height/2+toast.height));
                var moveto2 = new cc.MoveTo(0.02, cc.p(size.width/2, size.height/2-toast.height/2));
                var moveto3 = new cc.MoveTo(0.01, cc.p(size.width/2, size.height/2));
                seq = new cc.Sequence(fadein, moveto1, moveto2, moveto3, delay, remove, call);
            }
                break;
            case ToastActionType.Middle:
            {
                toast.setPosition(cc.p(size.width/2, size.height/2));
                toast.setScale(0);
                var scaleto = new cc.ScaleTo(0.3, 1);
                seq = new cc.Sequence(scaleto, delay, remove, call);
            }
                break;
            default:
                break;
        }
        toast.runAction(seq);
    },


    /***
     * 检测是否有相同的toast
     * @param text
     * @returns {boolean}
     */
    checkSameToast : function(text){
        for (var i = 0; i < this._toastList.length; ++ i){
            var toast = this._toastList[i];
            if (toast.getLabelText() == text){
                return true;
            }
        }

        return false;
    },

});

FYEngine.s_toastmanager = null;
FYEngine.ToastManager.getInstance = function(){
    if (!FYEngine.s_toastmanager){
        FYEngine.s_toastmanager = new FYEngine.ToastManager();
    }

    return FYEngine.s_toastmanager;
};

/****
 * 添加通用toast
 * 原谅我就是懒,呵呵
 * @param text string
 * @constructor
 */
FYEngine.commToastBg = '';//需要针对每个游戏做对应的设置
FYEngine.AddCommToast = function(text){
    FYEngine.ToastManager.getInstance().AddTextToToastList(FYEngine.commToastBg, text);
};

FYEngine.AddToastwithBg = function(bg, text){
    FYEngine.ToastManager.getInstance().AddTextToToastList(bg, text);
};

