/**
 * Created by mintao on 16/5/26.
 * 继承ClientHandler必须重载的函数：
 * sendDataMsg
 * handleDataMsg
 * onAccessEvent
 * 可能需要实现的重载接口：
 * onConnectEvent
 */


var eClientEvent = {
    EVENT_CONNECT_OK : 1,        // connect ok
    EVENT_CONNECT_FAIL : 2,      // connect failed
    EVENT_CONNECT_CLOSED : 3,    // socket close
    EVENT_CONNECT_DATA : 4,      // data notify
    EVENT_CONNECT_RECONN : 5,    // connect reconn
    EVENT_CONNECT_DATANEW : 6,      // data notify
};

var eClientState = {
    CLIENT_STATE_NONE : 0,
    CLIENT_STATE_CONNECTED : 1,
    CLIENT_STATE_DISCLOSE : 2,
    CLIENT_STATE_RECONN : 3,
    CLIENT_STATE_RECONNOK : 4, //之后有用
};

var ClientHandler = cc.Class.extend({
    _client: null,
    _state : 0,
    _recvBuf : null, //接受缓存
    _sendBuf : null, //发送缓存
    _bSendOk : true, //是否可以发送消息
    _sendQueue : [], //发送队列
    _heartBeat : 0, //心跳计数
    _heartBeatTime : 0.5, //心跳时间
    _needReConnect : true, //判断是否需要再次发送重连
    _reconnetCount : 0, //重连次数
    _reconnetMaxCount : 0,//最大重连次数
    _reconnetTime : 0, //重连时间
    _reconnetMaxTime : 0, //重连检测时间
    _bloginOk : false, //是否登录成功
    _canSendHeart : false,
    _canSendNormalMsg : false,

    ctor: function(){
        this._sendBuf = new ByteBuffer();
        this._sendBuf.initWriteBuffer();

        this._recvBuf = new ByteBuffer();
        this._recvBuf.initReadBuffer();

        this._reconnetCount = 0;
        this._canSendNormalMsg = false;
    },

    setCanSendHeart : function(bool){
        if(this._canSendHeart == bool){
            return;
        }
        this._canSendHeart = bool;
        cc.log('------设置可以发送心跳'+bool);
    },

    setClient : function(client){
        this._client = client;
    },

    getClient : function(){
        return this._client;
    },

    getClientId : function() {
        return this._client.getClientId();
    },

    setState : function(s){
        this._state = s;
    },

    getState : function(){
        return this._state;
    },

    setReconnetMaxCount : function(count){
        if (count <= 0){
            this._reconnetMaxCount = 1000;
        }else{
            this._reconnetMaxCount = count;
        }
    },

    setReconnetMaxTime : function(time){
        this._reconnetMaxTime = time;
    },

    checkReconnet : function(dt){
        this._reconnetTime += dt;
        //cc.log('--------this._reconnetTime:'+this._reconnetTime+'  :+this._reconnetMaxTime-1: '+this._reconnetMaxTime);
        if(this._reconnetTime > (this._reconnetMaxTime-1)){
            this.setCanSendHeart(false);
            cc.log('-------超时1');
        }
        if(this._reconnetTime > this._reconnetMaxTime){
            if( this._needReConnect == true ){
                this.reconnClient();

                this.onConnectEvent && this.onConnectEvent(eClientEvent.EVENT_CONNECT_RECONN);
                this._needReConnect = false;
            }
            this._reconnetTime = 0;
        }
    },

    setHeartBeatTime : function (time) {
        this._heartBeatTime = time;
    },

    checkSendHeartBeat : function(time){
        this._heartBeat += time;
        if (this._heartBeat >= this._heartBeatTime){
            this.sendHeartBeatReq();
            this._heartBeat = 0;
        }
    },

    setLoginStatus : function(state){
        this._bloginOk = state;
    },

    sendClientDataMsg : function(arrayBuf){
        if (this._client){
            this._client.sendDataMsg(arrayBuf);
        }
    },

    closeClient : function(){
        this.setState(eClientState.CLIENT_STATE_DISCLOSE);
        var client = this.getClient();
        if (client){
            this._sendQueue = [];
            client.stop();
            this.setClient(null);
        }
    },

    reconnClient : function(){
        var client = this.getClient();
        if( client != null ){
            cc.log('------------js重新链接Client-----');
            this.setLoginStatus(false);
            this._canSendNormalMsg = false;
            this._sendQueue = [];
            ++ this._reconnetCount;
            client.connect();
            this.setState(eClientState.CLIENT_STATE_RECONN);

        }
    },

    resetReconnetCount : function(){
        this._reconnetCount = 0;
        this._reconnetTime = 0;
    },

    /***
     * push send req
     * @param req
     */
    pushSendReq : function(req){
        this._sendQueue.push(req);
    },

    /***
     * send msg
     */
    onSendAllMsg : function(time){
        if (!this._bSendOk) {
            return;
        }

        if (this._bloginOk){
            if(this._canSendHeart){
                this.checkSendHeartBeat(time);
            }
            this.checkReconnet(time);
        }

        this._bSendOk = false;
        for (var i = 0; i < this._sendQueue.length; ++ i){
            this._sendBuf.initData();
            var req = this._sendQueue[i];
            switch (req.mainCmd){
                case CommonProtocol.LinkProto:
                    this.sendLinkMsg(req.subCmd,req);
                    break;
                case CommonProtocol.AccessProto:
                    this.sendAccessMsg(req.subCmd, req);
                    break;
                default:{
                    this.sendDataMsg(req.subCmd, req);
                }break;
            }
            this.sendClientDataMsg(this._sendBuf.getWriteBuffer());
        }
        this._sendQueue = [];
        this._bSendOk = true;
    },

    /***
     * receive msg
     * @param eventId
     * @param eventParam
     */
    onConnectionMsg : function(eventId, eventParam){
        switch(eventId){
            case eClientEvent.EVENT_CONNECT_OK:
            {
                cc.log('----------链接成功EVENT_CONNECT_OK-----clientId : '+this.getClientId());
                this.setState(eClientState.CLIENT_STATE_CONNECTED);
                this._sendQueue = [];
                this.sendPublicKeyRequest();
                this.onConnectEvent && this.onConnectEvent(eventId);
            }break;

            case eClientEvent.EVENT_CONNECT_FAIL:
            {
                //cc.log('-------超时2');
                this.setCanSendHeart(false);
                this._canSendNormalMsg = false;
                cc.log('----------链接失败EVENT_CONNECT_FAIL------clientId : '+this.getClientId());
                //cc.log('------this._reconnetCount---'+this._reconnetCount+'--- this._reconnetMaxCount---'+ this._reconnetMaxCount);
                this.onConnectEvent && this.onConnectEvent(eventId);
                if ((this.getState() == eClientState.CLIENT_STATE_CONNECTED ||
                    eClientState.CLIENT_STATE_RECONN == this.getState() ) &&
                    this._reconnetCount < this._reconnetMaxCount){
                    this.reconnClient();
                }
                else{
                    this.closeClient();
                }
            }break;

            case eClientEvent.EVENT_CONNECT_CLOSED:
                //this.reconnClient();
                this.onConnectEvent && this.onConnectEvent(eventId);
                break;

            case eClientEvent.EVENT_CONNECT_DATANEW:{
                this._recvBuf.initData();
                this._recvBuf.setReadBuffer(eventParam);

                var msgLen = this._recvBuf.getInt32();
                var mainCmd = this._recvBuf.getInt32();
                var subCmd  = this._recvBuf.getInt32();
                var msgId = this._recvBuf.getInt32();
                var route = this._recvBuf.getInt32();
                var msgType = this._recvBuf.getInt32();
                var encryptionType = this._recvBuf.getInt32();
                var checksum = this._recvBuf.getInt32();

                this.handleAllDataMsg(mainCmd,subCmd,this._recvBuf);
            }break;

            default:
                break;
        }
    },

    handleAllDataMsg : function(mainCmd, subCmd, byteBuf,msgType){
        switch (mainCmd){
            case CommonProtocol.AccessProto:
                this.handleAccessMsg(subCmd,byteBuf);
                break;
            case CommonProtocol.LinkProto:
                this.handleLinkMsg(subCmd,byteBuf);
                break;
            default:
                this.handleDataMsg(mainCmd, subCmd, byteBuf,msgType);
                this._reconnetTime = 0;
                break;
        }
    },

    /****
     * 链接消息
     * @param subCmd
     * @param req
     */
    sendLinkMsg : function(subCmd, req){
        switch (subCmd){
            case CommonProtocol.HeartBeat:
                CommonProtocol.encode_HeartBeat(req, this._sendBuf);
                break;
        }
    },

    sendHeartBeatReq : function(){
        var req = {};//createObj(CommonProtocol.HeartBeatReq);
        req.mainCmd = CommonProtocol.LinkProto;
        req.subCmd = CommonProtocol.HeartBeat;
        req.lRoleId = appglobal.getPlayer().roleid;
        this.pushSendReq(req);
       cc.log('------发送心跳------'+this.getClientId());
    },

    handleLinkMsg : function(subCmd,byteBuf){
        switch (subCmd){
            case CommonProtocol.HeartBeat:{
                cc.log('-------------收到心跳'+this.getClientId());
                this._reconnetTime = 0;
                this._heartBeat = 0;
            }
                break;
            default:
                break;
        }
    },

    /***
     * Access 消息
     * @param subCmd
     * @param byteBuf
     */
    sendAccessMsg : function(subCmd, req){
        switch (subCmd){
            case CommonProtocol.PublicKey:
                CommonProtocol.encode_PublicKey(req, this._sendBuf);
                break;
            case CommonProtocol.PrivateKey:{
                var client = this.getClient();
                if (client){
                    client.SetEncrytype(EntryptType.Entrypt_Rsa);
                    client.setPrvKey(appglobal.getPlayer().userid, req.dwPrvKeyLen, req.szPrvKey);
                    CommonProtocol.encode_PrivateKey(req, this._sendBuf);
                }
            }
                break;
        }
    },

    handleAccessMsg : function(subCmd,byteBuf){
        switch(subCmd){
            case CommonProtocol.PublicKey:
                this.handlePublicKeyRsp(byteBuf);
                break;
            case CommonProtocol.PrivateKey:
                this.handlePrivateKeyRsp(byteBuf);
                break;
            default:
                break;
        }
    },

    sendPublicKeyRequest : function(){
        var req = {};
        req.mainCmd = CommonProtocol.AccessProto;
        req.subCmd = CommonProtocol.PublicKey;
        req.iType = EntryptType.Entrypt_None;
        this.pushSendReq(req);
//       FYEngine.logObject(req,'---------发送公约:');
    },

    handlePublicKeyRsp : function(byteBuf){
        var rsp = CommonProtocol.decode_PublicKey(byteBuf);
//       FYEngine.logObject(rsp, '----------接受公钥:');
        var client = this.getClient();
        client.setPubKey(appglobal.getPlayer().userid, rsp.dwPubKeyLen, rsp.szPubKey);
        this._sendQueue = [];
        this.sendPrivateKeyRequest();

    },

    sendPrivateKeyRequest : function(){
        var req = {};
        req.mainCmd = CommonProtocol.AccessProto;
        req.subCmd = CommonProtocol.PrivateKey;
        req.dwPrvKeyLen = 32;
        var prvStr = '';
        for (var i = 0; i < 5; ++ i) {
            prvStr[i] = FYMath.getRandom(0, 100);///设置随机数
        }
        req.szPrvKey = hex_md5(prvStr).toUpperCase();
        this.pushSendReq(req);
//       FYEngine.logObject(req,'-----------发送私钥');
        
    },

    handlePrivateKeyRsp : function(byteBuf){
        var rsp = CommonProtocol.decode_PrivateKey(byteBuf);
//       FYEngine.logObject(rsp,'----------私钥');
        if (rsp.iResult == 0){
            this.getClient().SetEncrytype(EntryptType.Entrypt_Aes);
        }
        this.onAccessEvent(rsp.iResult);
    },


});


var ClientHandlerMgr = cc.Class.extend({
    handlers : null,

    ctor : function () {
        this.handlers = {};
    },

    addHandler : function(client_id,handler) {
        this.handlers[client_id] = handler;
        return handler;
    },

    removeHandler : function(client_id){
        delete this.handlers[client_id];
    },

    getHandler : function(client_id){
        return this.handlers[client_id];
    },

    getHandlers : function(){
        return this.handlers;
    }
});
